#-----------------------------------------------------------------------------
# Posing-to-New-Aux
# Jan 20,07
# Copyright (c) 2007 by David G. Drumright (ockham)
# The script will take the current actual position of the Posing cam
# and make a new Dolly cam in that place.

#-----------------------------------------------------------------------------

import poser
import string
import math
import os

scene = poser.Scene()
BasePathUncut=poser.AppLocation()
BasePath=BasePathUncut[:(-len('poser.exe'))]
FakeCR2Name=os.path.normpath(BasePath+'/runtime/libraries/character/fakecam.cr2')

#-----------------------------------------------------------------------------

# default vars
Type='AUX'  # or 'STD' for dolly
Name = 'Aux 2'
ParentName = 'UNIVERSE'
focal = 100.0
hither = 0.4
yon = 200.0
dollyZ = 2.0
dollyY = 1.0
dollyX = 0.0
scale = 1.0
zOrbit = 0.0
xOrbit = 0.0
yOrbit = 0.0

Fig = scene.CurrentFigure()
if not Fig: raise "No figure"

Name=Fig.Name() + '_AuxCam'

#-----------------------------------------------------------------------------
#InternalCreate=0
#if float(poser.Version()) >= 7.0:  InternalCreate=1

R2D=180.0 / math.pi

Posing=scene.Actor("Posing Camera")

# Locate the object of the Posing cam
scene.DrawAll() # for WV
PG=Posing.Geometry()
Vb=PG.WorldVertex(30) # back vert, by observation of the actual obj
Vf=PG.WorldVertex(22) # front vert
dX=Vf.X()-Vb.X()
dY=Vf.Y()-Vb.Y()
dZ=Vf.Z()-Vb.Z()
cx=(Vf.X()+Vb.X())/2.0
cy=(Vf.Y()+Vb.Y())/2.0
cz=(Vf.Z()+Vb.Z())/2.0
# For the xrot, figure a dHoriz from x and z.
dH=math.hypot(dX,dY)

# Set the dolly's tran at the center of the posing cam object
dollyX=(Vf.X()+Vb.X())/2.0
dollyY=(Vf.Y()+Vb.Y())/2.0
dollyZ=(Vf.Z()+Vb.Z())/2.0

# Set its angles by the arctan
yOrbit=  math.atan2(dX,dZ)
xOrbit= -math.atan2(dY,dH)
scale = Posing.ParameterByCode(poser.kParmCodeASCALE).Value()

#-----------------------------------------------------------------------------

fpout = open(FakeCR2Name,"wt")

fpout.write('{\nversion\n\t{\n\tnumber 4\n\t}\n') # Version 4

fpout.write('camera %s_CAMERA\n\t{\n\tname %s\n\t' % (Type,Name))

fpout.write('off\n\tbend 1\n\tdynamicsLock 1\n\thidden 0\n\taddToMenu 1\n\t')

fpout.write('castsShadow 1\n\tincludeInDepthCue 0\n\tuseZBuffer 1\n\tparent %s\n\t' % ParentName)

#===============

fpout.write('channels\n\t{\n\t') # Channels start

#==========
# focal
fpout.write('focal focal\n\t\t{\n\t\tname focal\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % focal)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# hither
fpout.write('hither hither\n\t\t{\n\t\tname hither\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % hither)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# yon
fpout.write('yon yon\n\t\t{\n\t\tname yon\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % yon)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# dollyZ
fpout.write('translateZ dollyZ\n\t\t{\n\t\tname dollyZ\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % dollyZ)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# dollyY
fpout.write('translateY dollyY\n\t\t{\n\t\tname dollyY\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % dollyY)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# dollyX
fpout.write('translateX dollyX\n\t\t{\n\t\tname dollyX\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % dollyX)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# scale
fpout.write('scale scale\n\t\t{\n\t\tname scale\n\t\tinitValue 100\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin 0\n\t\tmax 100000\n\t\ttrackingScale 0.1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % scale)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# Zrot
fpout.write('rotateZ roll\n\t\t{\n\t\tname zOrbit\n\t\tinitValue 0\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin -36000\n\t\tmax 36000\n\t\ttrackingScale 1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % zOrbit)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# Xrot
fpout.write('rotateX pitch\n\t\t{\n\t\tname xOrbit\n\t\tinitValue 0\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin -36000\n\t\tmax 36000\n\t\ttrackingScale 1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % xOrbit)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')
#==========
# Yrot
fpout.write('rotateY yaw\n\t\t{\n\t\tname yOrbit\n\t\tinitValue 0\n\t\t')

fpout.write('hidden 0\n\t\tforceLimits 1\n\t\tmin -36000\n\t\tmax 36000\n\t\ttrackingScale 1\n\t\t')

fpout.write('keys\n\t\t{\n\t\tstatic 0\n\t\tk 0 %f\n\t\t' % yOrbit)

fpout.write('sl 1\n\t\tspl\n\t\tsm\n\t\t}\n\t\tinterpStyleLocked 0\n\t}\n\t')

#==========
# End of channels
fpout.write('}\n\t')

#===============
# constant Stuff after channels

fpout.write('endPoint 0 1 0\n\torigin 0 0 0\n\torientation 0 0 0\n\tdisplayOrigin 0\n\t')

fpout.write('displayMode WIREFRAME\n\tcustomMaterial 0\n\tlocked 0\n\tbackfaceCull 0\n\t')

fpout.write('visibleInReflections 1\n\tvisibleInRender 1\n\tdisplacementBounds 0\n\t')

fpout.write('shadingRate 0.2\n\tsmoothPolys 0\n\tcameraModel poser\n\t}\n')

# Doc to add the camera

fpout.write('doc\n\t{\n\t{\n\taddCamera %s\n\t}\n}\n' % Name)

fpout.close()

#-----------------------------------------------------------------------------
# Now the fake CR2 is written. Load it:
scene.LoadLibraryFigure(FakeCR2Name)
scene.SetCurrentCamera(scene.Actor(Name))

#-----------------------------------------------------------------------------
